/**
 * Constants.
 */
const NS_ANNOTATION_METADATA_CID = Components.ID("{25d8aad5-4526-4cd6-8d94-0bddf8761339}");
const NS_ANNOTATION_METADATA_PROG_ID = "@mozilla.org/annotation-metadata;1";

function Metadata() {
  this.table = {};
}

Metadata.ANNOT_NS = "http://www.w3.org/2000/10/annotation-ns#";
Metadata.ANNOT_TYPE_NS = "http://www.w3.org/2000/10/annotationType#";
Metadata.RDF_NS   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
Metadata.DC_NS    = "http://purl.org/dc/elements/1.0/";
Metadata.THREAD_NS= "http://www.w3.org/2001/03/thread#";

Metadata.prototype.QueryInterface = function(iid) {
    if (!iid.equals(Components.interfaces.nsIAnnotationMetadata) &&
	!iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
    return this;
}


/**
 * Get the value of a metadata property.
 *
 * @param namespace string namespace for property
 * @param property string name of property
 * @return value for property or null if property does not exist
 */
Metadata.prototype.getProperty = function(namespace, property) {
  if (this.table[namespace + property]) {
    return this.table[namespace + property].value;
  }
  else {
      return null;
  }
}

/**
 * Set a Metadata property that is a literal ( a normal string ). 
 * If the property already exists, it will be overwritten.
 *
 * @param namespace string namespace for property
 * @param property string name of property 
 * @param value string value of property
 */
Metadata.prototype.setLiteralProperty = function(aNamespace, aProperty, aValue) {
    this.table[aNamespace + aProperty] = {
	type: MetadataProperty.LITERAL,
	namespace: aNamespace,
	property: aProperty,
	value: aValue };
}

/**
 * Set a Metadata property that is a resource ( a URI ). 
 * If the property already exists, it will be overwritten.
 * 
 * @param namespace string namespace for property
 * @param property string name of property 
 * @param uri string uri of resource
 */
Metadata.prototype.setResourceProperty = function(aNamespace, aProperty, aUri) {
    this.table[aNamespace + aProperty] = {
	type: MetadataProperty.RESOURCE,
	namespace: aNamespace,
	property: aProperty,
	value: aUri };
}

/**
 * Remove a metadata property.
 * If the property does not exist, 
 * an error will be thrown.
 *
 * @param namespace string namespace for property
 * @param property string property name
 * @exception if field does not exist
 */
Metadata.prototype.removeProperty = function(namespace, property) {
     if (this.table[namespace + property]) {
       delete this.table[namespace + property];
     }
     else {
	 throw Components.Exception("Metadata property " + namespace + property + " is not defined.");
     }
}

/**
 * Setup the required fields for annotation
 * metadata, as stipulated by the W3C Annotea
 * Project.
 *
 * @param 
 */
Metadata.prototype.setRequiredProperties = function (type, annotates, context, creator, created, date) {
 
    this.setLiteralProperty(Metadata.DC_NS, "creator", creator); 
    this.setLiteralProperty(Metadata.ANNOT_NS, "created", created);
    this.setLiteralProperty(Metadata.DC_NS, "date", date);
    this.setLiteralProperty(Metadata.ANNOT_NS, "context", context);
  
    this.setResourceProperty(Metadata.RDF_NS, "type", type);
    this.setResourceProperty(Metadata.ANNOT_NS, "annotates", annotates);
}

/**
 * Get all of the metadata properties
 * in an array. 
 *
 * @param count this is a required field for using arrays
 *              as parameters.  pass this as a blank object.
 * @param properties out parameter whose value field, upon
 *                   return, will have the array of nsIMetadataProperty
 */
Metadata.prototype.getAllProperties = function(count, a) {
    // copy meta over
    var list = [];
    for (prop in this.table) {
	var element = this.table[prop];
	if(element) {
	    list.push(new MetadataProperty(element.type, element.namespace, element.property, element.value));
	}
    }
    
    count.value = list.length;
    a.value = list;
}

/**
 * Returns a deep clone of this nsIAnnotationMetadata.
 *
 * @return deep clone of this
 */
Metadata.prototype.clone = function() {
  var clonedData = new Metadata();
  
  for (prop in this.table) {
    var property = this.table[prop];
    if (property.type == MetadataProperty.RESOURCE) {
      clonedData.setResourceProperty(property.namespace, property.property,
				     property.value);
    }
    else {
      clonedData.setLiteralProperty(property.namespace, property.property,
				    property.value);
    }
  }

  return clonedData.QueryInterface(Components.interfaces.nsIAnnotationMetadata);
}

/**
 * A Metadata property.
 */
function MetadataProperty(type, namespace, property, value) {
    this.mType = type;
    this.mNamespace = namespace;
    this.mProperty = property;
    this.mValue = value;
}

MetadataProperty.RESOURCE = 1;
MetadataProperty.LITERAL  = 2;

MetadataProperty.prototype = {
    get type() { return this.mType; },
    get namespace() { return this.mNamespace; },
    get property() { return this.mProperty; },
    get value() { return this.mValue; }
};
    
MetadataProperty.prototype.QueryInterface = function(iid) {
    if (!iid.equals(Components.interfaces.nsIMetadataProperty) &&
	!iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
    return this;
}


/********
 * XPCOM 
 * stuff
 ********/

var metadataModule = {
    firstTime: true,
    
    /*
     * RegisterSelf is called at registration time (component installation
     * or the only-until-release startup autoregistration) and is responsible
     * for notifying the component manager of all components implemented in
     * this module.  The fileSpec, location and type parameters are mostly
     * opaque, and should be passed on to the registerComponent call
     * unmolested.
     */
    registerSelf: function (compMgr, fileSpec, location, type) {
        if (this.firstTime) {
            dump("*** Deferring registration of nsAnnotationMetadata\n");
            this.firstTime = false;
            throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
        }
        dump("*** Registering nsAnnotationMetadata\n");
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(NS_ANNOTATION_METADATA_CID,
                                        "Annotation Metadata",
                                        NS_ANNOTATION_METADATA_PROG_ID, 
                                        fileSpec,
                                        location,
                                        type);
    },

    /*
     * The GetClassObject method is responsible for producing Factory and
     * SingletonFactory objects (the latter are specialized for services).
     */
    getClassObject: function (compMgr, cid, iid) {
        if (!cid.equals(NS_ANNOTATION_METADATA_CID)) {
	    throw Components.results.NS_ERROR_NO_INTERFACE;
	}

        if (!iid.equals(Components.interfaces.nsIFactory)) {
	    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	}

        return this.metadataFactory;
    },

    /* factory object */
    metadataFactory: {
        /*
         * Construct an instance of the interface specified by iid, possibly
         * aggregating it with the provided outer.  (If you don't know what
         * aggregation is all about, you don't need to.  It reduces even the
         * mightiest of XPCOM warriors to snivelling cowards.)
         */
        createInstance: function (outer, iid) {
            if (outer != null) {
		throw Components.results.NS_ERROR_NO_AGGREGATION;
	    }

            return (new Metadata()).QueryInterface(iid);
        }
    },

    /*
     * The canUnload method signals that the component is about to be unloaded.
     * C++ components can return false to indicate that they don't wish to be
     * unloaded, but the return value from JS components' canUnload is ignored:
     * mark-and-sweep will keep everything around until it's no longer in use,
     * making unconditional ``unload'' safe.
     *
     * You still need to provide a (likely useless) canUnload method, though:
     * it's part of the nsIModule interface contract, and the JS loader _will_
     * call it.
     */
    canUnload: function(compMgr) {
        dump("*** Unloading nsAnnotationMetadata\n");
        return true;
    }
};

function NSGetModule(compMgr, fileSpec) {
    return metadataModule;
}
